package com.docus.webservice.service;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.docus.webservice.entity.TBasic;
import com.docus.webservice.entity.TBasicExtend;
import com.docus.webservice.entity.TBasicSub;
import com.docus.webservice.handler.TBasicWebService;
import com.docus.webservice.mapper.TBasicExtendMapper;
import com.docus.webservice.mapper.TBasicMapper;
import com.docus.webservice.mapper.TBasicSubMapper;
import com.docus.webservice.mapper.TScanAssortMapper;
import com.docus.webservice.utils.ResultUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.sql.Wrapper;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 病案基本信息 服务实现类
 * </p>
 *
 * @author jiashi
 * @since 2021-04-14
 */
@Slf4j
@Service
public class TBasicServiceImpl extends ServiceImpl<TBasicMapper, TBasic> implements ITBasicService {

    @Autowired
    private TBasicMapper tBasicMapper;

    @Autowired
    private TBasicSubMapper tBasicSubMapper;

    @Autowired
    private TBasicExtendMapper tBasicExtendMapper;
    @Autowired
    private TScanAssortMapper tScanAssortMapper;

    private Logger log = LogManager.getLogger(TBasicServiceImpl.class);

    /**
     * 出院调用
     *
     * @param tBasic
     * @param tBasicSubMap
     * @param tBasicExtend
     */
    @Transactional
    @Override
    public void OutSavaAndSub(TBasic tBasic, HashMap<String, Object> tBasicSubMap, TBasicExtend tBasicExtend) {
        TBasic selectTBasic = tBasicMapper.selectOne(new QueryWrapper<TBasic>().eq("jzh", tBasic.getJzh()));
        if (selectTBasic == null) {
            log.info("基础信息住院天数----" + tBasic.getAdmissDays());
            int insert = tBasicMapper.insert(tBasic);
            if (insert <= 0) {
                throw new RuntimeException("数据库执行出错，请重试");
            }
            if (tBasicExtend.getClaimPolicyCode() != null || tBasicExtend.getMioSettleTypeCode() != null) {
                TBasic selectTBasic1 = tBasicMapper.selectOne(new QueryWrapper<TBasic>().eq("jzh", tBasic.getJzh()));
                tBasicExtend.setPatientId(selectTBasic1.getPatientId());
                TBasicExtend tBasicExtend1 = tBasicExtendMapper.selectOne(new QueryWrapper<TBasicExtend>().eq("patient_id", tBasicExtend.getPatientId()));
                if (tBasicExtend1 == null) {
                    int insert1 = tBasicExtendMapper.insert(tBasicExtend);
                    if (insert1 <= 0) {
                        throw new RuntimeException("医保结算类型/政策出错！");
                    }
                } else {
                    int i = tBasicExtendMapper.updateById(tBasicExtend);
                    if (i <= 0) {
                        throw new RuntimeException("医保结算类型/政策出错！");
                    }
                }
            }
            //TBasicSub tBasicSub = JSON.parseObject(JSON.toJSONString(tBasicSubMap), TBasicSub.class);
            // tBasicSub.setPatientId(tBasic.getPatientId());
            //insert = tBasicSubMapper.insert(tBasicSub);
            if (insert <= 0) {
                throw new RuntimeException("数据库执行出错，请重试");
            }
        } else {
            OutUpdateAndSub(tBasic, tBasicSubMap, tBasicExtend);
        }
    }

    /**
     * 出院调用
     *
     * @param tBasic
     * @param tBasicSubMap
     * @param tBasicExtend
     */
    @Override
    public void OutUpdateAndSub(TBasic tBasic, HashMap<String, Object> tBasicSubMap, TBasicExtend tBasicExtend) {
        long l = System.currentTimeMillis();
        TBasic selectTBasic = tBasicMapper.selectOne(new QueryWrapper<TBasic>().eq("jzh", tBasic.getJzh()));
        long l1 = System.currentTimeMillis();
        log.info("selectTBasic耗时---------" + (l1 - l));
        if (selectTBasic != null) {
            long l2 = System.currentTimeMillis();
            System.err.println(tBasic.getDisDate());
            log.info("基础信息住院天数----" + tBasic.getAdmissDays());
            LambdaUpdateWrapper<TBasic> eq = Wrappers.<TBasic>lambdaUpdate()
                    .set(TBasic::getDisDate, null == tBasic.getDisDate() ? null : tBasic.getDisDate())
                    .set(TBasic::getWardPalce, tBasic.getWardPalce()).set(TBasic::getAdmissDays, tBasic.getAdmissDays())
                    .set(TBasic::getAdmissDept, tBasic.getAdmissDept()).set(TBasic::getAdmissDeptName, tBasic.getAdmissDeptName())
                    .set(TBasic::getIsDead, tBasic.getIsDead()).set(TBasic::getName, tBasic.getName())
                    .set(TBasic::getInpatientNo, tBasic.getInpatientNo()).set(TBasic::getDisDept, tBasic.getDisDept())
                    .set(TBasic::getDisDeptName, tBasic.getDisDeptName()).eq(TBasic::getJzh, tBasic.getJzh());
            int update1 = tBasicMapper.update(tBasic, eq);
            if (tBasicExtend.getClaimPolicyCode() != null || tBasicExtend.getMioSettleTypeCode() != null) {
                tBasicExtend.setPatientId(selectTBasic.getPatientId());
                TBasicExtend selectTBasicExtend = tBasicExtendMapper.selectOne(new QueryWrapper<TBasicExtend>().eq("patient_id", tBasicExtend.getPatientId()));
                if (selectTBasicExtend == null) {
                    int insert1 = tBasicExtendMapper.insert(tBasicExtend);
                    if (insert1 <= 0) {
                        throw new RuntimeException("医保结算类型/政策出错！");
                    }
                } else {
                    int update = tBasicExtendMapper.update(tBasicExtend, new QueryWrapper<TBasicExtend>().eq("patient_id", selectTBasicExtend.getPatientId()));
                    if (update <= 0) {
                        throw new RuntimeException("医保结算类型/政策出错！");
                    }
                }
            }
            long l3 = System.currentTimeMillis();
            log.info("tBasicMapper.update耗时----------" + (l3 - l2));
            if (update1 <= 0) {
                throw new RuntimeException("数据库执行出错，请重试");
            }

            if (tBasicSubMap != null && tBasicSubMap.size() > 0) {
                long l4 = System.currentTimeMillis();
                TBasicSub tBasicSub = JSON.parseObject(JSON.toJSONString(tBasicSubMap), TBasicSub.class);
                long l5 = System.currentTimeMillis();
                log.info("JSON.parseObject耗时-------" + (l5 - l4));
                long l6 = System.currentTimeMillis();
                tBasicSub.setPatientId(selectTBasic.getPatientId());
                update1 = tBasicSubMapper.updateById(tBasicSub);
                if (update1 <= 0) {
                    throw new RuntimeException("数据库执行出错，请重试");
                }
            }
        } else {
            throw new RuntimeException("ResultUtils.fail(\"修改时,需提供有效的emp_id\");");
        }
    }

    @Override
    public void mergePreRecords(String startDate, String endDate) {
        log.info("预住院病历合并补偿开始---时间为----" + startDate + "----" + endDate);
        //页码
        int startrow;
        //每页10条数据
        final int size = 500;
        List<Map> patientIds;
        endDate = endDate + " 23:59:59";
        for (startrow = 0; ; startrow += size) {
            patientIds = tBasicMapper.selectByDisDateList(startDate, endDate,startrow,size);
            if (null == patientIds || patientIds.size() <= 0) {
                log.info("本次预住院病历合并结束！");
                break;
            }
            log.info("预住院病历合并补偿查询，startDate:{},endDate:{},startrow:{} size:{},总共:{}条数据",startDate, endDate, startrow, size, patientIds.size());
            for (Map patientId : patientIds) {
                try {
                    merge(patientId);
                } catch (Exception e) {
                    log.error(e.getMessage());
                }

            }
        }

    }

    @Override
    public void merge(Map<String, Object> patientId) {
        TBasicExtend tBasicExtend;
        TBasic tBasic;
        log.info(patientId.get("inpatient_no") + ":合并预住院病历开始");
        try {
            tBasicExtend = tBasicExtendMapper.selectOne(new QueryWrapper<TBasicExtend>().eq("patient_id", patientId.get("patient_id")));
            if (tBasicExtend != null && tBasicExtend.getPreJzh() != null) {
                tBasic = tBasicMapper.selectOne(new QueryWrapper<TBasic>().eq("jzh", tBasicExtend.getPreJzh()));
                if (tBasic != null) {
                    int i = tScanAssortMapper.updateByPatientId(String.valueOf(patientId.get("patient_id")), tBasic.getPatientId());
                    if (i <= 0) {
                        log.info("当前预住院没有文件！");
                    } else {
                        log.info(patientId.get("inpatient_no") + ":本次预住院合并" + i + "份报告");
                    }
                } else {
                    log.info(patientId.get("inpatient_no") + ":未查询到预住院信息！");
                }
            } else {
                log.info(patientId.get("inpatient_no") + ":未查询到预住院信息！");
            }
        } catch (Exception e) {
            log.error("预住院文件合并出错！-- 患者号: " + patientId.get("inpatient_no") + " 错误信息: " + e.getMessage(), e);
        }
    }
    /**
     * 住院调用
     *
     * @param tBasic
     * @param tBasicSubMap
     */
    @Override
    public void savaAndSub(TBasic tBasic, HashMap<String, Object> tBasicSubMap, String preJzh) {
        TBasic selectTBasic = tBasicMapper.selectOne(new QueryWrapper<TBasic>().eq("jzh", tBasic.getJzh()));
        if (selectTBasic == null) {
            tBasic.setFileSource(1);
            int insert = tBasicMapper.insert(tBasic);
            if (insert <= 0) {
                throw new RuntimeException("数据库执行出错，请重试");
            }
            //TBasicSub tBasicSub = JSON.parseObject(JSON.toJSONString(tBasicSubMap), TBasicSub.class);
            // tBasicSub.setPatientId(tBasic.getPatientId());
            //insert = tBasicSubMapper.insert(tBasicSub);
//            if (insert <= 0) {
//                throw new RuntimeException("数据库执行出错，请重试");
//            }
        } else {
            updateAndSub(tBasic, tBasicSubMap, preJzh);
        }
        if (null != preJzh && !preJzh.equals("")) {
            TBasic tbasic1 = tBasicMapper.selectOne(new QueryWrapper<TBasic>().eq("jzh", tBasic.getJzh()));
            TBasicExtend tBasicExtend1 = tBasicExtendMapper.selectOne(new QueryWrapper<TBasicExtend>().eq("patient_id", tbasic1.getPatientId()));

            if (null!=tbasic1) {
                TBasicExtend tBasicExtend = new TBasicExtend();
                if(null!=tBasicExtend1){
                    tBasicExtend1.setPreJzh(preJzh);
                    int update = tBasicExtendMapper.update(tBasicExtend1, new QueryWrapper<TBasicExtend>().eq("patient_id", tBasicExtend1.getPatientId()));
                    if (update <= 0) {
                        throw new RuntimeException("tBasicExtend修改出错，请重试");
                    }
                }else {
                    tBasicExtend.setPatientId(tbasic1.getPatientId());
                    tBasicExtend.setPreJzh(preJzh);
                    int insert = tBasicExtendMapper.insert(tBasicExtend);
                    if (insert <= 0) {
                        throw new RuntimeException("tBasicExtend新增出错，请重试");
                    }
                }


            }

        }
    }

    /**
     * 住院调用
     *
     * @param tBasic
     * @param tBasicSubMap
     */
    @Override
    public void updateAndSub(TBasic tBasic, HashMap<String, Object> tBasicSubMap, String preJzh) {
        long l = System.currentTimeMillis();
        TBasic selectTBasic = tBasicMapper.selectOne(new QueryWrapper<TBasic>().eq("jzh", tBasic.getJzh()));
        long l1 = System.currentTimeMillis();
        log.info("selectTBasic耗时---------" + (l1 - l));
        if (selectTBasic != null) {
            long l2 = System.currentTimeMillis();
            tBasic.setFileSource(1);
            int insert = tBasicMapper.update(tBasic, new QueryWrapper<TBasic>().eq("patient_id", selectTBasic.getPatientId()));
            long l3 = System.currentTimeMillis();
            log.info("tBasicMapper.update耗时----------" + (l3 - l2));
            if (insert <= 0) {
                throw new RuntimeException("数据库执行出错，请重试");
            }
            if (tBasicSubMap != null && tBasicSubMap.size() > 0) {
                long l4 = System.currentTimeMillis();
                TBasicSub tBasicSub = JSON.parseObject(JSON.toJSONString(tBasicSubMap), TBasicSub.class);
                long l5 = System.currentTimeMillis();
                log.info("JSON.parseObject耗时-------" + (l5 - l4));
                long l6 = System.currentTimeMillis();
                tBasicSub.setPatientId(selectTBasic.getPatientId());
                insert = tBasicSubMapper.updateById(tBasicSub);
                if (insert <= 0) {
                    throw new RuntimeException("数据库执行出错，请重试");
                }
            }
        } else {
            log.info("住院修改：患者" + tBasic.getJzh() + "不存在！\n新增患者" + tBasic.getJzh());
            savaAndSub(tBasic, tBasicSubMap, preJzh);
        }
    }
}
